'use strict';

var through = require('through2');
var istanbul = require('istanbul');

var instrumenter = new istanbul.Instrumenter({ preserveComments: true });

/**
 * Проверяет, нужно ли обрабатывать заданный файл
 * @function filterFunction
 * @param {Object} file
 * @param {String} file.path - полный путь к файлу
 * @returns {Boolean}
 */

/**
 * Инструментирует код для подсчета статистики покрытия кода тестами
 * @param {Object} [options] - параметры обработчика
 * @param {filterFunction} [options.filter] - функция, фильтрующая обрабатываемые файлы. Если не задана, обрабатываются все файлы.
 * @returns {Stream}
 */
module.exports = function () {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var filter = options.filter;


    !filter && (filter = function filter(obj) {
        return true;
    });

    return through.obj(function (file, encoding, cb) {

        try {
            if (filter(file)) {
                file.contents = instrumenter.instrumentSync(file.contents, file.path);
            }

            return cb(null, file);
        } catch (ex) {
            return cb(ex);
        }
    });
};